using DiningRoomContracts.BindingModels;
using DiningRoomContracts.BusinessLogicContracts;
using DiningRoomContracts.SearchModels;
using DiningRoomDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace DiningRoomView
{
    public partial class FormComponent : Form
    {
        private readonly ILogger _logger;
        private readonly IComponentLogic _logic;
        private int? _id;
        public int Id { set { _id = value; } }
        public int UserId { get; set; }
        public FormComponent(ILogger<FormComponent> logger, IComponentLogic logic)
        {
            InitializeComponent(); _logger = logger;
            _logic = logic;
        }
        private void FormComponent_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    _logger.LogInformation(" ");
                    var view = _logic.ReadElement(new ComponentSearchModel { Id = _id.Value, UserId = UserId });
                    if (view != null)
                    {
                        textBoxName.Text = view.ComponentName;
                        textBoxCost.Text = view.Cost.ToString();
                        comboBoxUnit.DisplayMember = "Unit";
                        comboBoxUnit.ValueMember = "Id";
                        comboBoxUnit.DataSource = Enum.GetValues(typeof(ComponentUnit)); ;
                        comboBoxUnit.SelectedItem = view.Unit;
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "  ");
                    MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                comboBoxUnit.DataSource = Enum.GetValues(typeof(ComponentUnit)); ;
                comboBoxUnit.SelectedItem = 1;
            }
        }
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show(" ", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation(" ");
            try
            {
                var model = new ComponentBindingModel
                {
                    Id = _id ?? 0,
                    ComponentName = textBoxName.Text,
                    Unit = (ComponentUnit)comboBoxUnit.SelectedItem,
                    Cost = Convert.ToDouble(textBoxCost.Text),
                    UserId = UserId
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("  .    .");
                }
                MessageBox.Show("  ", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "  ");
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
